--
-- CropDestruction
--
--
-- @author GIANTS Software
-- @date 15/10/2016
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

CropDestruction = {};

function CropDestruction.prerequisitesPresent(specializations)
    return true;
end

function CropDestruction:preLoad(savegame)
    self.loadDynamicWheelDataFromXML = Utils.overwrittenFunction(self.loadDynamicWheelDataFromXML, CropDestruction.loadDynamicWheelDataFromXML);
end

function CropDestruction:load(savegame)
    self.destroyFruitArea = CropDestruction.destroyFruitArea;

    local foundCareWheels = false;
    local storeItem = StoreItemsUtil.storeItemsByXMLFilename[self.configFileName:lower()];
    if storeItem.configurations ~= nil then
        if storeItem.configurations["wheel"] ~= nil then
            local wheelConfigurations = storeItem.configurations["wheel"];
            local config = wheelConfigurations[ self.configurations["wheel"] ];
            for _,value in pairs( { "configuration_valueTwinWheelsNarrow", "configuration_valueWheelNarrow" } ) do
                if config.name == g_i18n:getText(value) then
                    foundCareWheels = true;
                    break;
                end
            end
        end
    end;

    -- two hardcoded special cases, because the don't have a 'narrow wheel configuration set'
    if self.configFileName == "data/vehicles/steerable/newHolland/SP400F.xml" or
        self.configFileName == "data/vehicles/trailers/caruelleNicolas/caruelleNicolasStilla460.xml" or
        self.configFileName == "data/vehicles/trailers/bredal/K165.xml" then
        foundCareWheels = true;
    end

    self.cropDestruction = {};
    self.cropDestruction.isActive = not foundCareWheels and (self.wheels ~= nil and table.getn(self.wheels) > 0 and self.isServer);
end

function CropDestruction:loadDynamicWheelDataFromXML(superFunc, xmlFile, key, wheelnamei, wheel)
    if superFunc ~= nil then
        superFunc(self, xmlFile, key, wheelnamei, wheel);
    end

    local fallbackOldKey = "vehicle.wheels";

    wheel.useCropDestruction = Vehicle.getConfigurationValue(xmlFile, key, wheelnamei, "#useCropDestruction", getXMLBool, true, nil, fallbackOldKey);
end

function CropDestruction:delete()
end

function CropDestruction:mouseEvent(posX, posY, isDown, isUp, button)
end

function CropDestruction:keyEvent(unicode, sym, modifier, isDown)
end

function CropDestruction:update(dt)
    if not self.cropDestruction.isActive then
        return;
    end
    local rootAttacherVehicle = self:getRootAttacherVehicle();
    if rootAttacherVehicle.aiIsStarted then
        return;
    end
    if g_currentMission.fieldJobManager ~= nil and g_currentMission.fieldJobManager.currentFieldJob ~= nil then

        if g_currentMission.fieldJobManager.currentFieldJob.jobType == FieldJob.TYPE_FERTILIZING_GROWING or
            g_currentMission.fieldJobManager.currentFieldJob.jobType == FieldJob.TYPE_FERTILIZING_SOWN then

            for _,fieldJobVehicle in pairs(g_currentMission.fieldJobManager.currentFieldJob.fieldJobVehicles) do
                if self == fieldJobVehicle then
                    return;
                end
            end
        end
    end

    for _, wheel in pairs(self.wheels) do
        if wheel.useCropDestruction and wheel.contact == Vehicle.WHEEL_GROUND_CONTACT and wheel.hasTireTracks then

            -- limit size of destruction
            local width = 0.5 * wheel.width;
            local length = math.min(0.5, 0.5 * wheel.width);

            local x0,y0,z0;
            local x1,y1,z1;
            local x2,y2,z2;

            if wheel.repr == wheel.driveNode then
                x0,y0,z0 = localToWorld(wheel.node, wheel.positionX + width, wheel.positionY, wheel.positionZ - length);
                x1,y1,z1 = localToWorld(wheel.node, wheel.positionX - width, wheel.positionY, wheel.positionZ - length);
                x2,y2,z2 = localToWorld(wheel.node, wheel.positionX + width, wheel.positionY, wheel.positionZ + length);
            else
                local x,_,z = localToLocal(wheel.driveNode, wheel.repr, 0,0,0);
                x0,y0,z0 = localToWorld(wheel.repr, x + width, 0, z - length);
                x1,y1,z1 = localToWorld(wheel.repr, x - width, 0, z - length);
                x2,y2,z2 = localToWorld(wheel.repr, x + width, 0, z + length);
            end

            self:destroyFruitArea(x0,z0, x1,z1, x2,z2);

            if Vehicle.debugRendering then
                drawDebugLine(x0,y0,z0, 1,0,0, x1,y1,z1, 1,0,0);
                drawDebugLine(x0,y0,z0, 1,1,0, x2,y2,z2, 1,1,0);
            end

            if wheel.additionalWheels ~= nil then
                for _,additionalWheel in pairs(wheel.additionalWheels) do

                    local width = 0.5 * additionalWheel.width;
                    local length = math.min(0.5, 0.5 * additionalWheel.width);

                    local refNode = wheel.node;
                    if wheel.repr ~= wheel.driveNode then
                        refNode = wheel.repr;
                    end

                    local xShift,yShift,zShift = localToLocal(additionalWheel.wheelTire, refNode, 0,0,0);
                    local x0,y0,z0 = localToWorld(refNode, xShift + width, yShift, zShift - length);
                    local x1,y1,z1 = localToWorld(refNode, xShift - width, yShift, zShift - length);
                    local x2,y2,z2 = localToWorld(refNode, xShift + width, yShift, zShift + length);

                    self:destroyFruitArea(x0,z0, x1,z1, x2,z2);

                    if Vehicle.debugRendering then
                        drawDebugLine(x0,y0,z0, 1,0,0, x1,y1,z1, 1,0,0);
                        drawDebugLine(x0,y0,z0, 1,1,0, x2,y2,z2, 1,1,0);
                    end

                end
            end
        end
    end
end

function CropDestruction:updateTick(dt)
end

function CropDestruction:draw()
end

function CropDestruction:destroyFruitArea(x0,z0, x1,z1, x2,z2)
    local x, z, widthX, widthZ, heightX, heightZ = Utils.getXZWidthAndHeight(nil, x0,z0, x1,z1, x2,z2);

    for index, entry in pairs(g_currentMission.fruits) do

        local desc = FruitUtil.fruitIndexToDesc[index];
        if desc.allowsSeeding then
            local detailId = g_currentMission.terrainDetailId;

            if desc.preparingOutputName ~= nil and entry.preparingOutputId ~= nil then
                setDensityCompareParams(entry.preparingOutputId, "equal", 0);
                setDensityMaskParams(entry.preparingOutputId, "between", 2, desc.maxPreparingGrowthState+1);
                setDensityMaskedParallelogram(entry.preparingOutputId, x,z, widthX,widthZ, heightX,heightZ, 0, 1, entry.id, 0, g_currentMission.numFruitStateChannels, 1);
                setDensityCompareParams(entry.preparingOutputId, "greater", -1);
                setDensityMaskParams(entry.preparingOutputId, "greater", 0);

                setDensityCompareParams(entry.id, "between", 2, desc.maxPreparingGrowthState+1);
                setDensityParallelogram(entry.id, x,z, widthX,widthZ, heightX,heightZ, 0, g_currentMission.numFruitStateChannels, desc.cutState+1);
                setDensityCompareParams(entry.id, "greater", -1);
            --elseif desc.cutState < desc.minHarvestingGrowthState then
            elseif index == FruitUtil.FRUITTYPE_GRASS then
                -- grass
                setDensityCompareParams(entry.id, "between", desc.minHarvestingGrowthState+1, desc.maxHarvestingGrowthState+1);
                setDensityParallelogram(entry.id, x,z, widthX,widthZ, heightX,heightZ, 0, g_currentMission.numFruitStateChannels, desc.cutState+2);
                setDensityCompareParams(entry.id, "greater", -1);
            --elseif desc.minHarvestingGrowthState == desc.maxHarvestingGrowthState then
            elseif index == FruitUtil.FRUITTYPE_OILSEEDRADISH then
                -- oilseed, no destruction
            else
                -- all other/normal fruits
                setDensityCompareParams(entry.id, "greater", 2);
                setDensityMaskedParallelogram(entry.id, x,z, widthX,widthZ, heightX,heightZ, 0, g_currentMission.numFruitDensityMapChannels, detailId, g_currentMission.terrainDetailTypeFirstChannel, g_currentMission.terrainDetailTypeNumChannels, desc.cutState+1);
                setDensityCompareParams(entry.id, "greater", -1);
            end
        end
    end
end